#! /bin/sh
#
#   wipe-out.sh - インストール済みのFreeBSDで動く
#		  アプリケーションとしてのwipe-out
#
#	0.0: Jan.  2, 2017 by Dai ISHIJIMA
#

PATH=/usr/bin:/bin:/usr/sbin:/sbin:/usr/local/bin:/usr/local/sbin
export PATH

ttyname=`tty`
case x${ttyname} in
	x*ttyv*)
		echo "befor starting wipe-out on $ttyname," 1>&2
		echo "please enable Japanized console with kon" 1>&2
		exit 1
esac

if [ ! -x ../menu/disksize ]; then
	echo "check configured and current directory" 1>&2
	exit 1
fi

dir=`pwd`
PATH="${PATH}:$dir"
export WIPEOUTDIR=`echo $dir | sed 's/app$/menu/'`
PATH="${PATH}:$WIPEOUTDIR"
export PATH

export LOGFIL="${LOGFIL:-/tmp/wpoutlog.txt}"
export ERRORLOG="${ERRORLOG:-/tmp/wperrlog.txt}"
export DISPUPD="${DISPUPD:-2}"
export STATEFIL="${STATEFIL:-/tmp/wpstatus.txt}"
export DMESG="${DMESG:-/var/run/dmesg.boot}"
export TERM="${TERM:-vt102-color}"
export PATTERNFIL=/tmp/pattern.dat
export WIPEMODE=APP

echo '# -------------------------------------------------- #' >> $LOGFIL
echo -n 'Starting wipe-out at ' >> $LOGFIL
date >> $LOGFIL
echo '' >> $LOGFIL

stuff -g > $PATTERNFIL
echo "random pattern file $PATTERNFIL generated" >> $LOGFIL

# 認識するデバイス群
export DEVICES='da|ada|aacd|amrd|idad|ipsd|mfid|mlxd|twed|ad|ar'

# 一度に32Mバイトずつ消す
export BLKLEN=$(( 32 * 1024 * 1024 ))
# 出力ブロック長は1Mバイト
export OBS=$(( 1024 * 1024 ))
# 入力ブロック長 (stuffの出力長) は1kバイト
export IBS=1024
# 消去確認は10Mバイトずつ
export VBLKLEN=$(( 10 * 1024 * 1024 ))
# 消去確認の入力ブロック長は1Mバイト
export VBS=$(( 1024 * 1024 ))
# コピー時は512バイトずつ
export CBS=512

cd ../menu
sh mainmenu.sh
echo ''
echo ''
echo "'sh $0' to start wipe-out in application mode"

exit 0

# EOF
